/**************************************************************
* Project:      Repeater                                      *
* Descriptsion: Controls a HAM repeater with option for 1750Hz*
*               tone, CTCSS, wave file in beginding and end   *
*               of transmision and wave file bell will open   *
* Date:         19-09-2011                                    *
* Programmer:   5P6Q, Niels Krogsgaard                        *
***************************************************************
* Version:      1.1                                           *
* Ver. date:    18-12-2011                                    *
* Ver. program.:5P6Q, Niels Krogsgaard                        *
* Changes:      V.1.1 Timeout funktion added                  *   
**************************************************************/

#include <AF_Wave.h>
#include <avr/pgmspace.h>
#include "util.h"
#include "wave.h"


/*************************************************************
                     USER SETUP
**************************************************************/

boolean CTCSS_enable = true;       // Set to false if CTCSS is'nt in use
boolean tone_enable = true;        // Set to false if 1750 tone is'nt in use
char begin_wave[13] = "BEGIN.WAV"; // Filename for start audio file, max 8 characters + ".wav"
                                   // Begin_wave is only used if the repeater is started vith 1750 Hz tone
boolean begin_wave_enable = true;  // Set to false if no wave is used in begining                                   
char end_wave[13] = "END.WAV";     // Filename for end audio file, max 8 characters + ".wav"
                                   // end_wave is only used if the repeater is started vith 1750 Hz tone
boolean end_wave_enable = true;    // Set to false if no wave is used in end                               
char end_wave_CTCSS[13] = "ENDCTCSS.WAV"; // Filename for end audio file, max 8 characters + ".wav"
                                   // end_wave_CTCSS is only used if the repeater is started vith CTCSS
char bell_wave[13] = "BELL.WAV";   // Filename for end audio file, max 8 characters + ".wav"
                                   // If no bell is needed place a silend wave file
char nr_bell = 5;                  // Numbers of bells played before closing the repeater
boolean service_mode = false;      // Service mode send status and failes to the serial monitor
boolean Squels_enable = true;      // Select if the squels on the reciver can activate the transmitter
                                   // If set to false only CTCSS can activate the transmitter
                                   // WARNING!!! Do not disable this if tone is enabled
                                   // = = = = = = = = = = = = = = = = = = = = = = = = =
boolean low_squels = false;        // Set to false if squels input is activ high and true if squels input is activ low                                  
int tx_timeout = 300;              // Tx timeout in secundes (max time for 1 user to transmit nonestop)
                                   // MAX VALUE: 600
/*************************************************************
                 END USER SETUP
*************************************************************/

// Init sound- and SD-card

AF_Wave card;
File f;
Wavefile wave;
uint16_t samplerate;

// I/O pins

  int loop_on = 6;  // Output for switch reciver to transmitter (pin D6)
  int SC_on = 7;    // Output for switch soundcard to transmitter (pin D7)
  int SQ = 8;       // Input Squels activ from reciver (pin D8)
  int CTCSS = 9;    // Input CTCSS detected (pin D9)
  int PTT = 14;     // Output Key transiver (Pin A0)
  int CTCSS_OFF = 15; // Output disable transmitting CTCSS (Pin A1)
  int tone_on = 16;    // Input from 1750 Hz tone detector (Pin A2, Activ low)
  unsigned int timeout_counter = 0; // Counter for tx timeout 
  int x;

// Flags
boolean open_tone = false; // tells that the repeater was started by tone
boolean open_CTCSS = false; // tells that the repeater was started by CTCSS
boolean begin_played = false; // Tells if the begin wave file has been played
uint8_t i, r;
char c, name[13];

void setup()
{

// Setup I/O pins  
  pinMode(loop_on, OUTPUT);
  pinMode(SC_on, OUTPUT);
  pinMode(SQ, INPUT);
  pinMode(CTCSS, INPUT);
  pinMode(PTT, OUTPUT);
  pinMode(CTCSS_OFF, OUTPUT);
  pinMode(tone_on, INPUT);
// Setup I/O pins for soundcard
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  
  
  if(service_mode) Serial.begin(9600);  // Setup serial liberary to 9600 baud if service mode is on
  // Testing SD-card
  if (!card.init_card())
  {
    if(service_mode) Serial.println("Wave: Card init. failed");
    return;
  }
  if (!card.open_partition())
  {
    if(service_mode) Serial.println("Wave: No partition on card");
    return;
  }
  if(!card.open_filesys())
  {
    if(service_mode) Serial.println("Wave: Couldn't open filesys");
    return;
  }
  if(!card.open_rootdir())
  {
    if(service_mode) Serial.println("Wave: Couldn't open dir");
    return;
  }
  putstring_nl("Wave: Files found:");
  ls();
}

void ls() // Test of SD card
{
  int ret;
  card.reset_dir();
  while(1)
  {
    ret = card.get_next_name_in_dir(name);
    if (!ret)
    {
      card.reset_dir();
      return;
    }
  if(service_mode) Serial.println(name);
  }
}


void playfile (char *name) // Playing file from SD card
{
  f = card.open_file(name);
  if (!f)
  {
    if(service_mode) {Serial.print("Wave: Couldn't open file: "); Serial.println(name);}
    return;
  }
  if (!wave.create(f))
  {
    if(service_mode) {Serial.print("Wave: Not a valid WAVE file: ");Serial.println(name);}
    return;  
  }
  if(service_mode){Serial.print("Wave: playing file: ");Serial.println(name);}
  wave.play();
}



void audioswitch(byte radio, byte soundcard) // control audio from reciver and soundcard
{
  if (radio == 1)
  {
    if(service_mode)Serial.println("Audio: Audio from reciver transmitted");
    digitalWrite(loop_on, HIGH);
    digitalWrite(SC_on, LOW);
    digitalWrite(CTCSS_OFF, LOW);   // Transmit CTCSS tone when looping from reciver
  }
  else if (soundcard == 1)
  {
    if(service_mode)Serial.println("Audio: Audio from soundcard transmitted");
    digitalWrite(loop_on, LOW);
    digitalWrite(SC_on, HIGH);
    digitalWrite(CTCSS_OFF, HIGH);  // Disable CTCSS tone when sending from soundcard
  }
  else
  {
    if(service_mode)Serial.println("Audio: No audio transmitted");
    digitalWrite(loop_on, LOW);
    digitalWrite(SC_on, LOW);
    digitalWrite(CTCSS_OFF, HIGH);
  }  
}

void tonedetect()
{
  int x, tone_counter=0;
  if(!digitalRead(tone_on)) // if 1750Hz tone is detected start check loop
  {
    if(service_mode)Serial.println("1750: Tone detected");
    for(x=0;x<10;x++) // This loop detects that the tone is present for ½ sek before starting repeater
    {
      if(!digitalRead(tone_on))
        tone_counter++;
      delay(50);            // wait 50ms for next reading
    }
    if(service_mode){Serial.print("1750: Tone quallity: "); Serial.print(tone_counter); Serial.println("0%");}
    if(tone_counter >=7)    // if 1750 was detected min 70% of the time the last 500ms the repeater starts
    {
      if(service_mode)Serial.println("1750: Tone ack");
      open_tone = true;
    }
    else
    {
      if(service_mode)Serial.println("1750: Tone fault");
      open_tone = false;
    }  
    while(!digitalRead(tone_on)); // wait untill the tone is releast before opening the repeater
  }
  
}

void CTCSSdetect()
{
  if(digitalRead(CTCSS))  // check if CTCSS is recieved
  {
    delay(250);            // wait 250 ms and check again to remove errors
    if(digitalRead(CTCSS))
    {
      if(service_mode)Serial.println("CTCSS: CTCSS detected");
      open_CTCSS=true;
    }
  }
}

void repeat_signal() // repeat audio signal from reciver to transmitter
{
  timeout_counter = 0;
  if ((digitalRead(SQ)^low_squels) && Squels_enable)
    {  
    while ((digitalRead(SQ)^low_squels) && Squels_enable && (timeout_counter < (tx_timeout*100))) // If Squels from reciver and squels is enable and timeout not reaced
      {
        if(service_mode)Serial.println("Repeat: Repeating RX based on SQL");
        audioswitch(1,0);  // Loop recived auto to transmitter
        timeout_counter++;
        delay(10);  // 0.01 sec delay
      }
  }
  else while (digitalRead(CTCSS) && (timeout_counter < (tx_timeout*100))) // If CTCSS signal i recived
  {
    if(service_mode)Serial.println("Repeat: Repeating RX based on CTCSS");
    audioswitch(1,0);
    timeout_counter++;
    delay(10);  // 0.01 sec delay
  }
  audioswitch(0,0); // close audio loop
  if(timeout_counter >= (tx_timeout*100)) // If a user has been Tx longer the tx_timeout secunds
  {
    if(service_mode)Serial.println("Repeat: Stop by TX timeout");
    digitalWrite(PTT, LOW); // Stop transmitting
    x = 99; // Stop bell loop
    while (digitalRead(CTCSS)) {} // Makes sure that CTCSS users has stop transmitting before  the repeater can start again
  }
  timeout_counter = 0;  
    
}

void loop()
{
  uint16_t potval;
  uint32_t newsamplerate;
  
  
  while(open_tone==false && open_CTCSS==false) // waits for the repeater to be started
  {
    if(service_mode)Serial.println("Main: Standby mode");
    if(tone_enable)
      tonedetect();      // check for tone
    if(CTCSS_enable)
      CTCSSdetect();     // check for CTCSS
  }
  if(open_tone==true) // if repeater was started with 1750Hz tone
  {
    if(service_mode)Serial.println("Main: Started by 1750 Hz tone");

    if(service_mode)Serial.println("Main: Keying Transmitter");
    digitalWrite(PTT, HIGH); //key transmitter
    if((begin_wave_enable) && (begin_played==false))  // If start wave is enabled
    {
      begin_played = true;
      audioswitch(0,1);  //Enable audio from soundcard
      if(service_mode)Serial.println("Main: Playing: Start wave");
      card.reset_dir();
      playfile(begin_wave);  // play file
      samplerate = wave.dwSamplesPerSec;
      while (wave.isplaying)
      {
        if(digitalRead(SQ)^low_squels)  // Stop if signal recived
        {
          delay(250); // 250ms delay for noice remove
          if(digitalRead(SQ)^low_squels)
          {
           if(service_mode)Serial.println("Main: Wave Player interrupt by reciver");
           audioswitch(0,0); // disable audio from soundcard
           wave.stop();   // Stop playing file
           card.close_file(f);
           repeat_signal();
          }
        }
      }
      card.close_file(f); 
    }
    // Playing bells
    for(x=1;x<=nr_bell;x++)
    {
      audioswitch(0,1);  //Enable audio from soundcard
      if(service_mode) {Serial.print("Main; playing bell # "); Serial.println(x);}
      card.reset_dir();
      playfile(bell_wave);  // play file
      samplerate = wave.dwSamplesPerSec;
      while (wave.isplaying)
      {
        if(digitalRead(SQ)^low_squels)  // Stop if signal recived
        {
          delay(250); // noice filter
          if(digitalRead(SQ)^low_squels)
          {
            if(service_mode)Serial.println("Main: Wave Player interrupt by reciver");
            audioswitch(0,0); // disable audio from soundcard
            wave.stop();   // Stop playing file
            card.close_file(f);
            repeat_signal();
            x=0;              // reset bell counter
          }
        }
        
      }
      card.close_file(f); 
    }
    if(end_wave_enable)  // If end wave is enabled
    { 
      if(service_mode)Serial.println("Main: Playing end wave for 1750Hz tone start");
      audioswitch(0,1);  //Enable audio from soundcard
      card.reset_dir();
      playfile(end_wave);  // play file
      samplerate = wave.dwSamplesPerSec;
      while (wave.isplaying)
      {
        if(digitalRead(SQ)^low_squels)  // Stop if signal recived
        {
          delay(250); // nocie filter
          if(digitalRead(SQ)^low_squels)
          {
            if(service_mode)Serial.println("Main: Wave Player interrupt by reciver");
            audioswitch(0,0); // disable audio from soundcard
            wave.stop();   // Stop playing file
            card.close_file(f);
            repeat_signal();
            x=0; // repeat bell loop
          }
        }
      }
      card.close_file(f); 
    }
    if(x>0)
    {
    audioswitch(0,0);  //close audio
    open_tone=false; // reset repeater
    open_CTCSS=false;
    begin_played=false;
    }
  if(service_mode)Serial.println("Main: Stop keying Transmitter");
  digitalWrite(PTT, LOW); // stop key transmitter  
  }
  if(open_CTCSS)   // If repeater was started with CTCSS
  {
    if(service_mode)Serial.println("Main: Started by CTCSS");
     // Playing bells
    
    if(service_mode)Serial.println("Main: Keying Transmitter");
    digitalWrite(PTT, HIGH); //key transmitter
    
    if(((digitalRead(SQ)^low_squels)&&Squels_enable)||digitalRead(CTCSS))  // Repeat reviced signal
        {
          repeat_signal();
        }
    for(x=1;x<=nr_bell;x++)
    {
      audioswitch(0,1);  //Enable audio from soundcard
      if(service_mode) {Serial.print("Main: playing bell # "); Serial.println(x);}
      card.reset_dir();
      playfile(bell_wave);  // play file
      samplerate = wave.dwSamplesPerSec;
      while (wave.isplaying)
      {
        if(((digitalRead(SQ)^low_squels)&&Squels_enable)||digitalRead(CTCSS))  // Stop if signal recived
        {
          delay(250); // noice filter
          if(((digitalRead(SQ)^low_squels)&&Squels_enable)||digitalRead(CTCSS))
            {
            if(service_mode)Serial.println("Main: Wave Player interrupt by reciver");
            audioswitch(0,0); // disable audio from soundcard
            wave.stop();   // Stop playing file
            card.close_file(f);
            repeat_signal();
            x=0;              // reset bell counter
          }
        }
      }
      card.close_file(f); 
    }
    if(end_wave_enable)  // If end wave is enabled
    { 
      if(service_mode)Serial.println("Main: Playing end wave for CTCSS start");
      audioswitch(0,1);  //Enable audio from soundcard
      card.reset_dir();
      playfile(end_wave_CTCSS);  // play file
      samplerate = wave.dwSamplesPerSec;
      while (wave.isplaying)
      {
        if(((digitalRead(SQ)^low_squels)&&Squels_enable)||digitalRead(CTCSS))  // Stop if signal recived
        {
          delay(250); // noice filter
          if(((digitalRead(SQ)^low_squels)&&Squels_enable)||digitalRead(CTCSS))
          {
            if(service_mode)Serial.println("Main: Wave Player interrupt by reciver");
            audioswitch(0,0); // disable audio from soundcard
            wave.stop();   // Stop playing file
            card.close_file(f);
            repeat_signal();
            x=0;
          }
        }
      }
      card.close_file(f); 
    }
    if(x>0) // If the repeat loop wan't reset in end loop
    {
      audioswitch(0,0);  //close audio
      open_tone=false; // reset repeater
      open_CTCSS=false; 
      if(service_mode)Serial.println("Main: Stop keying Transmitter");
      digitalWrite(PTT, LOW); //stop key transmitter
    }
    
  }
}


